/*
 * (SLIK) SimpLIstic sKin functions
 * (C) 2001 John Ellis
 *
 * Author: John Ellis
 *
 * This software is released under the GNU General Public License (GNU GPL).
 * Please read the included file COPYING for more information.
 * This software comes with no warranty of any kind, use at your own risk!
 */

#include "blah.h"
#include "display.h"

#include "ui2_button.h"
#include "ui2_dial.h"
#include "ui2_item.h"
#include "ui2_main.h"
#include "ui2_number.h"
#include "ui2_slider.h"
#include "ui2_text.h"
#include "ui2_widget.h"

#include "ui_tabcomp.h"	/* for test */
#include "ui_utildlg.h"	/* for test */
#include "window.h"

/* glue between SLIK and rest of app */

/*
 *-----------------------------------------------------------------------------
 * display widgets, cbs, blah blah
 *-----------------------------------------------------------------------------
 */

/* buttons w/light */

static gint button_play_status_cb(ButtonData *button, const gchar *key, gpointer data)
{
	return TRUE;
}

static void button_play_click_cb(ButtonData *button, const gchar *key, gpointer data)
{
	printf("play pressed\n");
}

static void button_exit_click_cb(ButtonData *button, const gchar *key, gpointer data)
{
	blah_exit();
}

static void button_about_click_cb(ButtonData *button, const gchar *key, gpointer data)
{
	show_about();
}

static void change_skin_cancel_cb(FileDialog *fd, gpointer data)
{
	file_dialog_close(fd);
}

static void change_skin_load_cb(FileDialog *fd, gpointer data)
{
	ui_skin_load(main_window, fd->dest_path, NULL);
	file_dialog_sync_history(fd, FALSE);
	file_dialog_close(fd);
}

static void button_change_skin_click_cb(ButtonData *button, const gchar *key, gpointer data)
{
	FileDialog *fd;

	fd = file_dialog_new("Select skin - SLIK test", NULL, "slik_test", "select_skin", change_skin_cancel_cb, NULL);
	file_dialog_add_button(fd, "Load", change_skin_load_cb, TRUE);
	file_dialog_add_path_widgets(fd, NULL, "select_skin", "*", NULL);
	file_dialog_add_filter(fd, ".jpg; .png", "Graphics", FALSE);
	file_dialog_add_filter(fd, "skindata", "Skin Definition File", TRUE);
	gtk_widget_show(GENERIC_DIALOG(fd)->dialog);
}

static void button_editor_click_cb(ButtonData *button, const gchar *key, gpointer data)
{
	edit_window();
}

static gchar *text_hello_status_cb(TextData *text, const gchar *key, gpointer data)
{
	return "Hello world!";
}

static gchar *text_exit_status_cb(TextData *text, const gchar *key, gpointer data)
{
	return "Exit:";
}

static gchar *text_about_status_cb(TextData *text, const gchar *key, gpointer data)
{
	return "About:";
}

static gchar *text_change_status_cb(TextData *text, const gchar *key, gpointer data)
{
	return "Change:";
}

static gchar *text_editor_status_cb(TextData *text, const gchar *key, gpointer data)
{
	return "Editor:";
}

void display_register_widgets(UIData *ui)
{
	button_register_key("play", ui,
			    button_play_status_cb, NULL,
			    button_play_click_cb, NULL,
			    NULL, NULL,
			    NULL, NULL);
	button_register_key("exit", ui,
                            NULL, NULL,
                            button_exit_click_cb, NULL,
                            NULL, NULL,
                            NULL, NULL);
	button_register_key("about", ui,
			    NULL, NULL,
			    button_about_click_cb, NULL,
			    NULL, NULL,
			    NULL, NULL);

	button_register_key("change_skin", ui,
			    NULL, NULL,
			    button_change_skin_click_cb, NULL,
			    NULL, NULL,
			    NULL, NULL);
	button_register_key("skin_editor", ui,
			    NULL, NULL,
			    button_editor_click_cb, NULL,
			    NULL, NULL,
			    NULL, NULL);

	text_register_key("hello", ui, text_hello_status_cb, NULL);
	text_register_key("exit", ui, text_exit_status_cb, NULL);
	text_register_key("about", ui, text_about_status_cb, NULL);
	text_register_key("change", ui, text_change_status_cb, NULL);
	text_register_key("editor", ui, text_editor_status_cb, NULL);
}

/*
 *-----------------------------------------------------------------------------
 * display widget update functions (public)
 *-----------------------------------------------------------------------------
 */


void display_set_hello_text(const gchar *text)
{
	text_set_text("hello", main_window, (gchar *)text);
	text_set_autoscroll("hello", main_window, FALSE);
}



