/*
 * (SLIK) SimpLIstic sKin functions
 * (C) 2001 John Ellis
 *
 * Author: John Ellis
 *
 * This software is released under the GNU General Public License (GNU GPL).
 * Please read the included file COPYING for more information.
 * This software comes with no warranty of any kind, use at your own risk!
 */

#include "blah.h"

#include "display.h"
#include "rcfile.h"
#include "ui2_editor.h"
#include "ui2_main.h"
#include "ui_fileops.h"
#include "ui_tabcomp.h"
#include "window.h"

static gint update_timeout_id = -1;

/*
 *-----------------------------------------------------------------------------
 * the about window
 *-----------------------------------------------------------------------------
 */

static GtkWidget *about = NULL;

static gint about_delete_cb(GtkWidget *widget, GdkEventAny *event, gpointer data)
{
	gtk_widget_destroy(about);
	about = NULL;

	return TRUE;
}

void show_about(void)
{
	GtkWidget *frame;
	GtkWidget *vbox;
	GtkWidget *label;
	GtkWidget *pixmap;
	GtkWidget *cred;
	GtkStyle *style;

	GdkPixmap *pmap;
	GdkBitmap *mask;

	gchar *buf;

	if (about)
		{
		gdk_window_raise(about->window);
		return;
		}

	about = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_signal_connect(GTK_OBJECT(about), "delete_event", GTK_SIGNAL_FUNC(about_delete_cb), NULL);

	gtk_window_set_title(GTK_WINDOW(about), "About - SLIK test");
	gtk_container_border_width(GTK_CONTAINER(about), 5);

	frame = gtk_frame_new(NULL);
	gtk_container_border_width(GTK_CONTAINER(frame), 5);
	gtk_container_add(GTK_CONTAINER(about), frame);
	gtk_widget_show(frame);

	vbox = gtk_vbox_new(FALSE, 0);
	gtk_container_border_width(GTK_CONTAINER(vbox), 5);
	gtk_container_add(GTK_CONTAINER(frame), vbox);
	gtk_widget_show(vbox);

	gtk_widget_realize(about);

	style = gtk_widget_get_style(about);
	pmap = gdk_pixmap_create_from_xpm_d(about->window, &mask,
		&style->bg[GTK_STATE_NORMAL], (gchar **)ui_slik_logo());
	pixmap = gtk_pixmap_new(pmap, mask);
	gdk_pixmap_unref(pmap);
	if (mask) gdk_bitmap_unref(mask);

	gtk_box_pack_start(GTK_BOX(vbox), pixmap, FALSE, FALSE, 0);
	gtk_widget_show(pixmap);

	buf = g_strdup_printf("SLIK hello world\n%s\nCopyright (c) 2001 by John Ellis\nhttp://netpedia.net/hosting/gqview\ngqview@email.com\n\nReleased under the GNU Public License", VERSION);
	label = gtk_label_new(buf);
	g_free(buf);
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
	gtk_widget_show (label);

	cred = ui_slik_credit();
	gtk_box_pack_end(GTK_BOX(vbox), cred, FALSE, FALSE, 0);
	gtk_widget_show(cred);

	window_set_icon(about, ui_slik_logo(), NULL);
	gtk_widget_show(about);
}



/*
 *-----------------------------------------------------------------------------
 * the main loop! called 8 times per second (125 ms)
 *-----------------------------------------------------------------------------
 */

static gint update_condition(gpointer data)
{
	if (debug_mode) printf("main loop\n");

	return TRUE;
}

/*
 *-----------------------------------------------------------------------------
 * misc start-up utils
 *-----------------------------------------------------------------------------
 */

static void keys_load(void)
{
	gchar *path;

	path = g_strconcat(homedir(), "/", BLAH_RC_DIR, "/", PATH_HISTORY_NAME, NULL);
	history_list_load(path);
	g_free(path);
}

static void keys_save(void)
{
	gchar *path;

	path = g_strconcat(homedir(), "/", BLAH_RC_DIR, "/", PATH_HISTORY_NAME, NULL);
	history_list_save(path);
	g_free(path);
}

static void check_for_home_path(gchar *path)
{
	gchar *buf;
	buf = g_strconcat(homedir(), "/", path, NULL);
	if (!isdir(buf))
		{
		printf(_("Creating dir:%s\n"), buf);
		if (mkdir (buf, 0755) < 0)
			{
			printf(_("Could not create dir:%s\n"), buf);
			}
		}
	g_free(buf);
}

static gchar *determine_skin_path(gchar *file)
{
	gchar *path = NULL;
	gchar *buf;

	if (isdir(file))
		{
		path = g_strdup(file);
		}
	else
		{
		buf = g_strconcat(get_current_dir(), "/", file, NULL);
		if (isdir(buf))
			{
			path = buf;
			}
		else
			{
			g_free(buf);
			buf = g_strconcat(homedir(), "/", 
					BLAH_RC_DIR_SKIN, "/", file, NULL);
			if (isdir(buf))
				{
				path = buf;
				}
			else
				{
				g_free(buf);
				buf = g_strconcat(BLAH_SKINDIR, "/", file, NULL);
				if (isdir(buf))
					{
					path = buf;
					}
				else
					{
					g_free(buf);
					}
				}
			}
		}

	if (path)
		{
		if (debug_mode) printf (_("Using skin: %s\n"), path);
		}
	else
		{
		printf (_("Skin not found: %s\n"), file);
		}

	return path;
}

/*
 *-----------------------------------------------------------------------------
 * help function
 *-----------------------------------------------------------------------------
 */

static void show_help(void)
{
	printf("slik_test version %s\n", VERSION);
	printf(_("usage: slik_test [option(s)]\n"));
	printf(_(" standard options:\n"));
	printf(_("  --skin:<skindir>             load specified skin\n"));
	printf(_("  --skinhelp                   print mouse event coordinates\n"));
	printf(_("  --debug                      enable debug output\n"));
	printf(_("  -h,  --help                  displays this message\n"));
	printf(_("  --crack                      reserved\n"));
}

/*
 *-----------------------------------------------------------------------------
 * exit function
 *-----------------------------------------------------------------------------
 */

void blah_exit(void)
{
	save_options();
	keys_save();

	gtk_main_quit();
}

static gint schedule_exit_cb(gpointer data)
{
	blah_exit();
	
	return FALSE;
}

void blah_schedule_exit(void)
{
	gtk_idle_add(schedule_exit_cb, NULL);
}

/*
 *-----------------------------------------------------------------------------
 * main routine
 *-----------------------------------------------------------------------------
 */

int main (int argc, char *argv[])
{
	const gchar *cbuf;
	gchar *buf;
	gchar *startup_skin_file = NULL;
	gint i;
	gint tc = 0;

	/* setup locale, i18n */
	gtk_set_locale();
	bindtextdomain (PACKAGE, LOCALEDIR);
	textdomain (PACKAGE);

	/* setup globals (eww, well, so what) */

	/* check that we can find a valid home directory, if not exit semi-gracefully */
	cbuf = homedir();
	if (!cbuf)
		{
		printf(_("Unable to determine the home directory\n"));
		return 0;
		}
	else if (!isdir(cbuf))
		{
		printf(_("Could not find home directory: %s\n"), cbuf);
		return 0;
		}

	/* create dirs */
	check_for_home_path(BLAH_RC_DIR);
	check_for_home_path(BLAH_RC_DIR_SKIN);

	if (argc > 1)
		{
		i = 1;
		while (i < argc)
			{
			if (!strcmp(argv[i],"--debug"))
				{
				debug_mode = TRUE;
				printf("debugging enabled\n");
				}
			else if (!strcmp(argv[i],"--skinhelp"))
				{
				debug_skin = TRUE;
				printf("skin debugging enabled\n");
				}
			else if (!strncmp(argv[i],"--skin:", 7))
				{
				if (startup_skin_file)
					{
					printf(_("Ignoring multiple skins!\n"));
					}
				else
					{
					startup_skin_file = determine_skin_path(argv[i] + 7);
					}
				}
			else if (!strcmp(argv[i],"-h") || !strcmp(argv[i],"--help"))
				{
				show_help();
				return 0;
				}
			else if (!strcmp(argv[i],"--crack"))
				{
				tc++;
				}
			else
				{
				printf(_("unknown option: %s\nuse --help for options\n"),argv[i]);
				return 0;
				}
			i++;
			}
		}

/* */
	/*
	 * the gtk inits are moved after the command line parser
	 * (for speed with command line options, or errors thereof)
	 */
	gtk_init (&argc, &argv);

	gdk_rgb_init();

	/* push the correct color depths to gtk, (for 8-bit psuedo color displays)
	 * they should be popped, too, I guess...
	 */
	gtk_widget_push_visual(gdk_rgb_get_visual());
	gtk_widget_push_colormap(gdk_rgb_get_cmap());

	/* parse a custom gtkrc */
	buf = g_strconcat(homedir(), "/", BLAH_RC_DIR, "/gtkrc", NULL);
	if (isfile(buf)) gtk_rc_parse(buf);
	g_free(buf);

	/* load options */
	load_options();
	keys_load();

	/* register base dirs to editor */
	buf = g_strconcat(homedir(), "/", BLAH_RC_DIR_SKIN, NULL);
	ui_edit_add_skin_resource(buf);
	g_free(buf);
	ui_edit_add_skin_resource(BLAH_SKINDIR);

	/* set up main window */
	main_window = window_main_new(startup_skin_file);
	g_free(startup_skin_file);
	startup_skin_file = NULL;

	gtk_widget_show(main_window->window);

	while (tc > 0)
		{ 
		tc--;
		ui_distort(main_window);
		}

	/* start main loop */
	update_timeout_id = gtk_timeout_add(125, (GtkFunction)update_condition, NULL);

	gtk_main ();
	return 0;
}







