/*
 * (SLIK) SimpLIstic sKin functions
 * (C) 2001 John Ellis
 *
 * Author: John Ellis
 *
 * This software is released under the GNU General Public License (GNU GPL).
 * Please read the included file COPYING for more information.
 * This software comes with no warranty of any kind, use at your own risk!
 */

#include "ui2_includes.h"
#include "ui2_typedefs.h"
#include "ui2_default.h"

#include "ui2_skin.h"
#include "ui2_button.h"
#include "ui2_item.h"
#include "ui2_number.h"
#include "ui2_slider.h"
#include "ui2_text.h"
#include "ui2_util.h"
#include "ui2_widget.h"


#include "default_skin/back.xpm"
#include "default_skin/btn_config.xpm"
#include "default_skin/btn_exit.xpm"
#include "default_skin/btn_play.xpm"
#include "default_skin/letters.xpm"
#include "ui2_logo.xpm"

/* NOTE: some functions are #if'ed out to make compile with -Werror work
 */


static void add_item(const gchar *key, gchar **data, gint sections, gint x, gint y, SkinData *skin, const gchar *text_id)
{
	ItemData *item;

	item = item_new_from_data(data, sections, x, y);
	item_register(skin, item, key, text_id);
}

#if 0
static DigitData *add_digit(const gchar *key, gchar **data, SkinData *skin, const gchar *text_id)
{
	DigitData *digit;

	digit = digit_new_from_data(data);
	digit_register(skin, digit, key, text_id);
	digit_unref(digit);

	return digit;
}

static void add_number(const gchar *key, gint length, gint zeros, gint centered, gint x, gint y,
		       DigitData *digit, SkinData *skin, const gchar *text_id)
{
	NumberData *number;

	number = number_new(digit, length, zeros, centered, x, y);
	number_register(skin, number, key, text_id);
}
#endif

static WidgetData *add_button(const gchar *key, gchar **data, gint prelight, gint light, gint x, gint y,
			      gchar **clip_data, SkinData *skin, const gchar *text_id)
{
	ButtonData *button;

	button = button_new_from_data(data, prelight, light, x, y, clip_data);
	return button_register(skin, button, key, text_id);
}

#if 0
static void add_slider(const gchar *key, gchar **data, gint has_press, gint prelight, gint vertical, gint reversed,
		       gint length, gint x, gint y, SkinData *skin, const gchar *text_id)
{
	SliderData *slider;

	slider = slider_new_from_data(data, has_press, prelight, vertical, reversed, length, x, y, NULL);
	slider_register(skin, slider, key, text_id);
}
#endif

static void add_text(const gchar *key, gchar **data, gint length, gint extended, gint x, gint y,
	      SkinData *skin, const gchar *text_id)
{
	TextData *text;

	text = text_new_from_data(data, length, extended, x, y);
	text_register(skin, text, key, text_id);
}

SkinData *skin_load_default(const gchar *key)
{
	SkinData *skin;
#if 0
	DigitData *ds, *dl;
	WidgetData *wd;
#endif

	skin = skin_new();

	skin->real_overlay = util_size_pixbuf(gdk_pixbuf_new_from_xpm_data((const char **)back_xpm), TRUE);

	skin->width = gdk_pixbuf_get_width(skin->real_overlay);
	skin->height = gdk_pixbuf_get_height(skin->real_overlay);

	add_button("exit", (gchar **)btn_exit_xpm, TRUE, FALSE, 109, 4, NULL, skin, NULL);
	add_button("about", (gchar **)btn_config_xpm, TRUE, FALSE, 109, 22, NULL, skin, NULL);
	add_button("change_skin", (gchar **)btn_config_xpm, TRUE, FALSE, 109, 40, NULL, skin, NULL);
	add_button("skin_editor", (gchar **)btn_config_xpm, TRUE, FALSE, 109, 58, NULL, skin, NULL);
	add_button("play", (gchar **)btn_play_xpm, TRUE, TRUE, 48, 98, NULL, skin, NULL);

	add_text("hello", (gchar **)letters_xpm, 12, FALSE, 31, 80, skin, NULL);

	add_text("exit", (gchar **)letters_xpm, 8, FALSE, 52, 4, skin, NULL);
	add_text("about", (gchar **)letters_xpm, 8, FALSE, 52, 22, skin, NULL);
	add_text("change", (gchar **)letters_xpm, 8, FALSE, 52, 40, skin, NULL);
	add_text("editor", (gchar **)letters_xpm, 8, FALSE, 52, 58, skin, NULL);

	add_item("SLIK", (gchar **)slik_xpm, 1, 4, 4, skin, NULL);

	/* this is merely to that the skin editor has sane values for these */
	skin->width_def = skin->width_min = skin->width_max = skin->width;
	skin->height_def = skin->height_min = skin->height_max = skin->height;

	return skin;
}


