/*
 * (SLIK) SimpLIstic sKin functions
 * (C) 2001 John Ellis
 *
 * Author: John Ellis
 *
 * This software is released under the GNU General Public License (GNU GPL).
 * Please read the included file COPYING for more information.
 * This software comes with no warranty of any kind, use at your own risk!
 */

#include "ui2_includes.h"
#include "ui2_typedefs.h"
#include "ui2_util.h"

#include "ui_pixbuf_ops.h"

#include <gdk/gdkx.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>


/*
 * try 0.5 or 3 ;)
 * for non-whole numbers change GDK_INTERP_NEAREST to GDK_INTERP_BILINEAR below
 */
#define DOUBLE_MULTIPLIER 2


void util_size(gint *n)
{
	if (slik_double_size) *n *= DOUBLE_MULTIPLIER;
}

GdkPixbuf *util_size_pixbuf(GdkPixbuf *pb, gint apply_adjustments)
{
	if (pb && slik_double_size)
		{
		GdkPixbuf *tmp = pb;
		pb = gdk_pixbuf_scale_simple(tmp, gdk_pixbuf_get_width(tmp) * DOUBLE_MULTIPLIER,
					     gdk_pixbuf_get_height(tmp) * DOUBLE_MULTIPLIER, GDK_INTERP_NEAREST);
		gdk_pixbuf_unref(tmp);
		}

	if (pb && apply_adjustments && slik_transparency_force)
		{
		gint t;
		if (!gdk_pixbuf_get_has_alpha(pb))
			{
			GdkPixbuf *tmp = pb;
			pb = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8,
					    gdk_pixbuf_get_width(pb), gdk_pixbuf_get_height(pb));
			pixbuf_copy_area(tmp, 0, 0, pb, 0, 0,
					 gdk_pixbuf_get_width(pb), gdk_pixbuf_get_height(pb), FALSE);
			pixbuf_alpha_adjust(pb, 0, 0, gdk_pixbuf_get_width(pb), gdk_pixbuf_get_height(pb), 255);
			gdk_pixbuf_unref(tmp);
			}
		t = (gint)((float)-64.0 * (255 - slik_transparency_force_a) / 255);
		pixbuf_alpha_adjust(pb, 0, 0, gdk_pixbuf_get_width(pb), gdk_pixbuf_get_height(pb), t);
		}

	if (pb && apply_adjustments && slik_colorshift_on)
		{
		pixbuf_draw_rect_fill(pb, 0, 0, gdk_pixbuf_get_width(pb), gdk_pixbuf_get_height(pb),
				      slik_colorshift_r, slik_colorshift_g, slik_colorshift_b, slik_colorshift_a);
		}

	return pb;
}

/* 
 * returns a portion of the root window in a gdk_pixbuf
 * any part that is off the screen is set to black.
 */

GdkPixbuf *util_pixbuf_from_root_window(gint x, gint y, gint w, gint h, gint get_all)
{
	GdkPixbuf *pb;
	GdkVisual *gdkvisual;
	GdkWindow *rootwindow;
	gint screen_w;
	gint screen_h;
	gint screen_x;
	gint screen_y;


	if (w < 1 || h < 1 ) return NULL;

	pb = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8, w, h);
	pixbuf_draw_rect_fill(pb, 0, 0, w, h, 0, 0, 0, 255);

	screen_w = gdk_screen_width();
	screen_h = gdk_screen_height();

	if (x >= screen_w || y >= screen_h || x + w < 0 || y + h < 0) return pb;

	screen_x = x;
	if (screen_x < 0)
		{
		w += screen_x;
		screen_x = 0;
		}
	if (screen_x + w > screen_w)
		{
		w -= screen_x + w - screen_w;
		}

	screen_y = y;
	if (screen_y < 0)
		{
		h += screen_y;
		screen_y = 0;
		}
	if (screen_y + h > screen_h)
		{
		h -= screen_y + h - screen_h;
		}

	if (w < 1 || h < 1) return pb;

	rootwindow = (GdkWindow *) &gdk_root_parent;;
	if (!rootwindow) return NULL;

	gdkvisual = gdk_window_get_visual(rootwindow);
	if (gdkvisual != gdk_visual_get_system()) return pb;

	if (debug_mode) printf("root window capture: %d, %d (%d x %d) to %d, %d\n", screen_x, screen_y, w, h, x, y);

	if (!get_all)
		{
		/*
		 * get the backing pixmap
		 * Don't even ask how this works, I slapped it together
		 * from various references from other apps that do the same.
		 */
		Atom prop, type;
		int format;
		unsigned long length, after;
		unsigned char *data = NULL;
		Window desktop_window;

		gdk_error_trap_push();

		desktop_window = GDK_ROOT_WINDOW();

		prop = XInternAtom(GDK_DISPLAY(), "_XROOTPMAP_ID", True);
  
		if (prop != None)
			{
			XGetWindowProperty(GDK_DISPLAY(), desktop_window, prop, 0L, 1L, False,
					   AnyPropertyType, &type, &format, &length, &after,
					   &data);

			if (type == XA_PIXMAP)
				{
				Pixmap p;
				p = *((Pixmap *)data);

				if (p != None)
					{
					GdkPixmap *pp;
					GdkColormap *cmap;
					gint p_w, p_h;

					pp = gdk_pixmap_foreign_new(p);
					cmap = gdk_window_get_colormap(rootwindow);

					gdk_window_get_size(pp, &p_w, &p_h);

					if (p_w < screen_x + w || p_h < screen_y + h)
						{
						/* tiled */
						gint i, j;
						
						for (j = (screen_y / p_h) * p_h; j < screen_y + h; j += p_h)
						    for (i = (screen_x / p_w) * p_w; i < screen_x + w; i += p_w)
							{
							gint offset_x, offset_y;
							gint offset_w, offset_h;

							if (j < screen_y)
								{
								offset_y = screen_y - j;
								}
							else
								{
								offset_y = 0;
								}

							offset_h = p_h - offset_y;
							if (j + offset_y + offset_h >= screen_y + h) offset_h = (screen_y + h) - (j + offset_y);

							if (i < screen_x)
								{
								offset_x = screen_x - i;
								}
							else
								{
								offset_x = 0;
								}

							offset_w = p_w - offset_x;
							if (i + offset_x + offset_w >= screen_x + w) offset_w = (screen_x + w) - (i + offset_x);

							gdk_pixbuf_get_from_drawable(pb, pp, cmap,
										     offset_x, offset_y,
									     	     (i + offset_x) - screen_x, (j + offset_y) - screen_y, offset_w, offset_h);
							}
						}
					else
						{
						gdk_pixbuf_get_from_drawable(pb, pp, cmap,
									     screen_x, screen_y, 
								     	     screen_x - x, screen_y - y, w, h);
						}
					g_free(pp);
					}
				}
			if (data) XFree(data);
			}
		gdk_error_trap_pop();
		}
	else
		{
		/* merely get whatever is on the screen in this area, including other windows */
		gdk_pixbuf_get_from_drawable(pb, rootwindow, NULL,
					     screen_x, screen_y, 
					     screen_x - x, screen_y - y, w, h);
		}

	return pb;
}







