/*
 * (SLIK) SimpLIstic sKin functions
 * (C) 2001 John Ellis
 *
 * Author: John Ellis
 *
 * This software is released under the GNU General Public License (GNU GPL).
 * Please read the included file COPYING for more information.
 * This software comes with no warranty of any kind, use at your own risk!
 */

#include "blah.h"
#include "window.h"

#include "display.h"
#include "ui2_display.h"
#include "ui2_editor.h"
#include "ui2_main.h"

#include <gdk/gdkkeysyms.h> /* for key values */

void edit_window(void)
{
	EditData *ed;

	ed = ui_edit_new(main_window, "slik_test", "skin_editor", NULL);

	/* ed is NULL if an edit window is already active for the ui */
	if (!ed) return;
	ui_edit_set_help_path(ed, BLAH_SLIK_HELP_PATH);
}

static void key_pressed(GtkWidget *widget, GdkEventKey *event)
{
	static gint last_key = 0;
	static gint last_state = 0;
	static guint32 last_time = 0;

	/* cheap hacks/checks to avoid problems caused by holding down
	 * keys with fast repeat rates */
	if (event->keyval == last_key && event->state == last_state &&
			last_time + 500 >= event->time)
		{
		return;
		}
	else
		{
		last_key = event->keyval;
		last_state = event->state;
		last_time = event->time;
		}

	if (debug_mode) printf("key pressed: %d\n", event->keyval);

	if (event->state & GDK_CONTROL_MASK)
		{
		switch (event->keyval)
			{
			default:
				break;
			}
		}
	else
		{
		switch (event->keyval)
			{
			case GDK_Escape:
				printf ("Escape pressed\n");
				break;
			case '!':
				ui_debug_print_all_keys(main_window);
				break;
			case '~': case '`':
				edit_window();
				break;
			default:
				break;
			}
		}
}

static gint window_delete_cb(GtkWidget *widget, GdkEventAny *event, gpointer data)
{
	blah_exit();

	return TRUE;
}

void window_main_set_title(const gchar *title)
{
	if (main_window)
		{
		gchar *buf;

		if (title)
			{
			buf = g_strconcat(title, " - SLIK test", NULL);
			}
		else
			{
			buf = g_strdup("BLAH");
			}

		ui_title_set(main_window, buf);
		g_free(buf);
		}
}

static void window_main_mouse_cb(UIData *ui, gint button, guint32 time, gpointer data)
{
	printf("button %d pressed\n", button);
}

UIData *window_main_new(const gchar *skin_path)
{
	UIData *ui;

	ui = ui_new("SLIK_test", "main", wm_decorations, "SLIK test");
	ui_set_icon(ui, ui_slik_logo(), NULL);

	gtk_signal_connect (GTK_OBJECT (ui->window), "delete_event", (GtkSignalFunc) window_delete_cb, ui);
	gtk_signal_connect (GTK_OBJECT (ui->window), "key_press_event",(GtkSignalFunc) key_pressed, ui);

	ui_set_mouse_callback(ui, window_main_mouse_cb, NULL);

	/* register the right stuff */
	display_register_widgets(ui);

	if (skin_path)
		{
		printf(_("loading skin..\n"));
		if (!ui_skin_load(ui, skin_path, NULL))
			{
			if (skin_default_path)
				{
				if (!ui_skin_load(ui, skin_default_path, skin_mode_key))
					{
					printf(_("Default skin not loaded, using built-in default\n"));
					ui_skin_load(ui, NULL, NULL);
					}
				}
			else
				{
				printf(_("Skin not loaded, using built-in default\n"));
				ui_skin_load(ui, NULL, NULL);
				}
			}
		}
	else if (skin_default_path)
		{
		if (!ui_skin_load(ui, skin_default_path, skin_mode_key))
			{
			printf(_("Skin in config file not loaded, using built-in default\n"));
			ui_skin_load(ui, NULL, NULL);
			}
		}
	else
		{
		ui_skin_load(ui, NULL, skin_mode_key);
		}

	return ui;
}



